function [NrOfN NrOfNN NrOfC NrOfCCC] = ClassifyNodes(BinaryImg)
%Find an initial point on the contour (if the shape is not terminated, then choose an
%random point. Trace the contour and mark the connectivity (Gradient Image). Then classify
%each point as N, NN, C and CCC (See Kopolwitz and Bruckstein's paper)
% in Ron version there are no printouts

%---------- AUTHOR:
%Hamid Sarve 2006-06-27
 

%find init points
[y x] = find(BinaryImg == 1);

NrOfPixels = size(x,1);

%shape exists?
if(NrOfPixels < 1)
    error('no shape in the image');
end

%init canditates
cand_init_x = [];
cand_init_y = [];

index = 0; %candidate index
%search for a pixel without two neighbours 
for i=1:NrOfPixels,
    Kernel      = KernelNxM_const(BinaryImg,y(i),x(i),3,3,0);
    if(sum(sum(Kernel)) == 2)
       index         = index + 1;
       cand_init_x(index) = x(i);
       cand_init_y(index) = y(i);
    end
end

init_x = x(1);
init_y = y(1);

terminatedShape = true; %terminated shape means that the shape doesn't have a start and end point

if  (index > 0)
    init_x = cand_init_x(1);
    init_y = cand_init_y(1);
    terminatedShape = false;
end


if(terminatedShape) 
%     sprintf('shape is terminated')
else
%     sprintf('shape is not terminated')
end

% sprintf('Start point is chosen as x=%d, y=%d',init_x,init_y)

 
new_x = init_x;
new_y = init_y;

GradientImg   = zeros(size(BinaryImg,1),size(BinaryImg,2));
ContourImg    = BinaryImg;


%----------------------------------------------- Marking connectivity

for i=1:NrOfPixels-1,
    
   Kernel                = KernelNxM_const(ContourImg,new_y,new_x,3,3,0);
   Kernel(2,2)           = 0;
   [offset_y offset_x]   = find(Kernel == 1);


%  Direction-setting:  4 bits
%   N S E W
%   example
%   NW:
%   N S E W
%   0 1 0 1 = 5
 


   if(offset_y(1) == 1 && offset_x(1) == 1) direction = 9;  end
   if(offset_y(1) == 1 && offset_x(1) == 2) direction = 8;  end
   if(offset_y(1) == 1 && offset_x(1) == 3) direction = 10; end

   if(offset_y(1) == 2 && offset_x(1) == 1) direction = 1;  end
   if(offset_y(1) == 2 && offset_x(1) == 2) direction = 0;  end
   if(offset_y(1) == 2 && offset_x(1) == 3) direction = 2;  end

   if(offset_y(1) == 3 && offset_x(1) == 1) direction = 5;  end
   if(offset_y(1) == 3 && offset_x(1) == 2) direction = 4;  end
   if(offset_y(1) == 3 && offset_x(1) == 3) direction = 6;  end
 

   ContourImg(new_y,new_x)  = 0;
   GradientImg(new_y,new_x) =  direction;
   new_y = new_y + offset_y(1)-2;
   new_x = new_x + offset_x(1)-2;
 
end



%Use Bruckstein's algorithm for estimation of the perimeter----------------


 [x y] = find(GradientImg > 0);

NrOfPixels = size(x,1); 
%Bruckstein classification
NrOfNN  = 0;
NrOfN   = 0;
NrOfC   = 0;
NrOfCCC = 0;
 

PreviousGradient = GradientImg(init_y,init_x);
%debugging purpose WalkedPathImage = zeros(size(GradientImg,1),size(GradientImg,2));
 

new_y = init_y;
new_x = init_x;
for i=1:NrOfPixels,

    
    Kernel          = KernelNxM_const(GradientImg,new_y,new_x,3,3,0);
    GradientValue   = GradientImg(new_y,new_x);
 
%get the next offset
    if(GradientValue == 9)  offset_y=-1; offset_x=-1; end
    if(GradientValue == 8)  offset_y=-1; offset_x=0;  end
    if(GradientValue == 10) offset_y=-1; offset_x=1;  end
    if(GradientValue == 1)  offset_y=0;  offset_x=-1; end
    if(GradientValue == 2)  offset_y=0;  offset_x=1;  end
    if(GradientValue == 5)  offset_y=1;  offset_x=-1; end
    if(GradientValue == 4)  offset_y=1;  offset_x=0;  end
    if(GradientValue == 6)  offset_y=1;  offset_x=1;  end
    
    if( (sum(bitget(GradientValue,1:4)) > 1)  && sum(bitget(PreviousGradient,1:4)) > 1) 
       NrOfCCC = NrOfCCC + 1; 
       NrOfC   = NrOfC   + 1;
    else
        if( (sum(bitget(GradientValue,1:4)) < 2)  && sum(bitget(PreviousGradient,1:4)) < 2)
           NrOfNN = NrOfNN + 1; 
        else
            if( ((sum(bitget(GradientValue,1:4)) < 2)  && sum(bitget(PreviousGradient,1:4)) > 1)) 
                NrOfC = NrOfC + 2; 
            else
                if( (sum(bitget(GradientValue,1:4)) > 1)  && sum(bitget(PreviousGradient,1:4)) < 2)
                   NrOfN = NrOfN + 1; 
                end
            end
        end  
    end
    
 

    % - debugging purpose: WalkedPathImage(new_y,new_x) = 1;
    
    PreviousGradient = GradientValue;
    new_y = new_y + offset_y; 
    new_x = new_x + offset_x; 
end
 
clear BinaryImg;
clear GradientImg;
clear ContourImg;