function [Perimeter,imshowImg] = EstimatePerimeter(BinaryImg_in)
%Estimates the perimeter of a single shape, using the Koplowitz & Bruckstein's
%algorithm. (Design of Perimeter Estimators for Digitized Planar Shapes)
% changes I made include commenting the outputs and adding the new
% segmented image as an output so that we can show what was actually
% measured rather than what was the original input.

%---------- INPUT
%The input image shall be a binary image (if not: it will be converted
%using a threshold of 0.5) of the contour (8-connectivity) of a single shape with no
%'branches'. If any branches exist they will be removed.

%---------- OUTPUT
% Estimated Perimeter

%---------- PLEASE NOTE:
%Please note: Two estimator are presented in Koplowitz and Bruckstein's
%paper. This one is the implementation of the first one, referred as the so called
%"(c,cc,nn,nnnn)-estimator" in the paper

%---------- AUTHOR:
%Hamid Sarve 2006-06-28
%Centre for Image Analysis, Uppsala, Sweden


%Extend the input image with one pixel
BinaryImg = zeros(size(BinaryImg_in,1)+2,size(BinaryImg_in,2)+2);
BinaryImg(2:size(BinaryImg_in,1)+1,2:size(BinaryImg_in,2)+1) = BinaryImg_in ;
 
ImgWidth  = size(BinaryImg,2); %get the image size
ImgHeight = size(BinaryImg,1); %get the image size

%check if the input image is a binary image
%if not, convert it to a binary image
info = whos('BinaryImg'); class = info.class; isBinary = strcmp(class,'logical');
if(~isBinary)
    warning('the input image is not a binary image');
    BinaryImg = im2bw(BinaryImg,0.5);
    warning('setting the input image to a binary image (threshold 0.5)');
end
 



% %Correct connectivity ----------------------------------------------------
% % find the problematic element, add the pixels that it will erode to the
% % keepPix image and the ones on the side to the deletePix image
% % there are four types of "problematic" SEs to correct
keepPixels=zeros(size(BinaryImg));
deletePixels=zeros(size(BinaryImg));
% 
SEproblem=[1 0 0;0 1 1; 0 1 0];
apperanceImg = and(imerode(BinaryImg,SEproblem),BinaryImg) ;
keepPixels=apperanceImg;
[apperancePixelsRow,apperancePixelsCol]=find(apperanceImg);
for loop=1:length(apperancePixelsRow),
    deletePixels(apperancePixelsRow(loop),apperancePixelsCol(loop)+1)=1;
end;

SEproblem=[0 1 0;0 1 1; 1 0 0];
apperanceImg = and(imerode(BinaryImg,SEproblem),BinaryImg) ;
keepPixels=or(keepPixels,apperanceImg);
[apperancePixelsRow,apperancePixelsCol]=find(apperanceImg);
for loop=1:length(apperancePixelsRow),
    deletePixels(apperancePixelsRow(loop),apperancePixelsCol(loop)+1)=1;
end;

SEproblem=[0 1 0;0 1 1; 1 0 0];
SEproblem=fliplr(SEproblem);
apperanceImg = and(imerode(BinaryImg,SEproblem),BinaryImg) ;
keepPixels=or(keepPixels,apperanceImg);
[apperancePixelsRow,apperancePixelsCol]=find(apperanceImg);
for loop=1:length(apperancePixelsRow),
    deletePixels(apperancePixelsRow(loop),apperancePixelsCol(loop)-1)=1;
end;

SEproblem=[1 0 0;0 1 1; 0 1 0];
SEproblem=fliplr(SEproblem);
apperanceImg = and(imerode(BinaryImg,SEproblem),BinaryImg) ;
keepPixels=or(keepPixels,apperanceImg);
[apperancePixelsRow,apperancePixelsCol]=find(apperanceImg);
for loop=1:length(apperancePixelsRow),
    deletePixels(apperancePixelsRow(loop),apperancePixelsCol(loop)-1)=1;
end;


 
%% this added block is to solve the problem of crashing with the offset
%% empty because of an x shaped perimeter
% I decide arbitrarily to delete the pixel that goes diagonally
SEproblem=[0 0 1;0 1 0; 1 0 1];
apperanceImg = and(imerode(BinaryImg,SEproblem),BinaryImg) ;
% keepPixels=apperanceImg;
[apperancePixelsRow,apperancePixelsCol]=find(apperanceImg);
for loop=1:length(apperancePixelsRow),
    deletePixels(apperancePixelsRow(loop)-1,apperancePixelsCol(loop)+1)=1;
end;

SEproblem=[0 0 1;0 1 0; 1 0 1];
SEproblem=fliplr(SEproblem);
apperanceImg = and(imerode(BinaryImg,SEproblem),BinaryImg) ;
% keepPixels=apperanceImg;
[apperancePixelsRow,apperancePixelsCol]=find(apperanceImg);
for loop=1:length(apperancePixelsRow),
    deletePixels(apperancePixelsRow(loop)+1,apperancePixelsCol(loop)+1)=1;
end;


SEproblem=[0 0 1;0 1 0; 1 0 1];
SEproblem=flipud(SEproblem);
apperanceImg = and(imerode(BinaryImg,SEproblem),BinaryImg) ;
% keepPixels=apperanceImg;
[apperancePixelsRow,apperancePixelsCol]=find(apperanceImg);
for loop=1:length(apperancePixelsRow),
    deletePixels(apperancePixelsRow(loop)-1,apperancePixelsCol(loop)-1)=1;
end;


SEproblem=[0 0 1;0 1 0; 1 0 1];
SEproblem=fliplr(SEproblem);
SEproblem=flipud(SEproblem);
apperanceImg = and(imerode(BinaryImg,SEproblem),BinaryImg) ;
% keepPixels=apperanceImg;
[apperancePixelsRow,apperancePixelsCol]=find(apperanceImg);
for loop=1:length(apperancePixelsRow),
    deletePixels(apperancePixelsRow(loop)+1,apperancePixelsCol(loop)-1)=1;
end;



%remove undesired connectivity, i.e. an L-shape
SE         = {[0 0 0;1 1 0; 0 1 0],[0 1 0;1 1 0 ; 0 0 0],[0 0 0;0 1 1;0 1 0],[0 1 0;0 1 1; 0 0 0]};
for i=1:4,
    BinaryImg = and(~imerode(BinaryImg,SE{i}),BinaryImg) ;
end

% add the desired pixels

 BinaryImg=or(BinaryImg,keepPixels);
 BinaryImg=and(BinaryImg,~deletePixels);
 

%Just include the largest shape, exlude all others------------------------
LabledImage = bwlabel(BinaryImg,8);
NrOfShapes  = max(max(LabledImage));
MaxRegionSize = 0;
RegionIndex   = 0;
if(NrOfShapes > 1)
    warning('More than 1 shape in the image. Keeping the largest shape');
    for i=1:NrOfShapes,
        RegionSize = sum(sum(find(LabledImage == i)));
        if(RegionSize > MaxRegionSize)
            MaxRegionSize = RegionSize;
            RegionIndex   = i;
        end
    end
    BinaryImg(find(LabledImage ~= RegionIndex)) = 0;
end
clear LabledImage;
clear NrOfShapes;
clear MaxRegionSize;


%Correct the shape--------------------- ----------------------------------
%remove the branches, keep the main region

BinaryImg = RemoveBranches(BinaryImg);



 
%Classification of the nodes----------------------------------------------
[NrOfN NrOfNN NrOfC NrOfCCC] = ClassifyNodesRon(BinaryImg);
 


%Estimation using several algorithms, however only return Brucksteins-----
%Bruckstein's optimal weights
Perimeter = 0.99350*NrOfNN + 0.70251*NrOfCCC + 1.06681*NrOfN + 0.57736*NrOfC;

ChessBoardPerimeter = NrOfN + NrOfNN + NrOfCCC/2 + NrOfC / 2;
ManhattanPerimeter  = NrOfN + NrOfNN + NrOfCCC + NrOfC;

% sprintf('Perimeter Estimations: Chessboard = %d, Manhattan = %d, Bruckstein = %d',floor(ChessBoardPerimeter),floor(ManhattanPerimeter),round(Perimeter))

%---- Make  a fancy outprint
% clf;
imshowImg        = zeros(ImgHeight-2,ImgWidth-2,3);
imshowImg(:,:,1) =  BinaryImg(2:ImgHeight-1,2:ImgWidth-1); 
imshowImg(:,:,2) =  BinaryImg_in;
imshowImg(:,:,3) =  BinaryImg(2:ImgHeight-1,2:ImgWidth-1); 

% imshow(imshowImg);

% title('\fontsize{16} Length estimated for the marked white path.');
% text(5,10,'\color[rgb]{0.2 0.75 0.2} Green paths (if any) are ignored paths (in case the shape did not meet the constrains)');
 
% axis on;

clear BinaryImg; %no use anymore
clear BinaryImg_in;