function BinaryImg = RemoveBranches(BinaryImg)
%removes the branches of a contour, keeps the main region
%i.e.
%         X X X 
%    X X X 
%         X X X X
%
% is converted to
%         
%    X X X 
%         X X X X
%
%Please note: Branches are defined as the smaller path at a
%crosspoint


%---------- AUTHOR:
%Hamid Sarve 2006-06-24


ImgWidth  = size(BinaryImg,2); %get the image size
ImgHeight = size(BinaryImg,1); %get the image size

[y x] = find(BinaryImg == 1);


NrOfPixels  = size(x,1);
RegionIndex = 0;
Branches = 0;

%Remove the crosses,i.e. there're two branches at the same point----------   
se = [0 1 0; 1 0 1; 0 1 0];
Crosses = imerode(BinaryImg,se);
[crossPosY crossPosX] = find(Crosses == 1);

for i=1:size(crossPosY,1),
   
    MaskedImg   = zeros(ImgHeight,ImgWidth);
    MaskedImg   = BinaryImg;        

    MaskedImg(crossPosY(i)-1:crossPosY(i)+1,crossPosX(i)-1:crossPosX(i)+1) = 0;

    LabledImg = bwlabel(MaskedImg);
    NrOfShapes  = max(max(LabledImg));
    MinRegionSize = Inf;
    if(NrOfShapes > 1)
        Branches = Branches + 1;
        for i=1:NrOfShapes,
            RegionSize = sum(sum(find(LabledImg == i)));
            if(RegionSize <= MinRegionSize)
                MinRegionSize = RegionSize;
                RegionIndex   = i;
            end

        end
        se=[0 1 0; 1 1 1; 0 1 0];
        LabledImg = imdilate(LabledImg,se);
        BinaryImg(find(LabledImg == RegionIndex)) = 0;
 
    end

end

%remove single branches---------------------------------------------
for i=1:NrOfPixels,
        Kernel      = KernelNxM_const(BinaryImg,y(i),x(i),3,3,0);
        if (sum(sum(Kernel)) > 3)

            MaskedImg   = zeros(ImgHeight,ImgWidth);
            MaskedImg   = BinaryImg;        
            MaskedImg(y(i),x(i)) = 0;
            
            LabledImg = bwlabel(MaskedImg);
            NrOfShapes  = max(max(LabledImg));
            MinRegionSize = Inf;
            if(NrOfShapes > 1)
                Branches = Branches + 1;
                for i=1:NrOfShapes,
                        RegionSize = sum(sum(find(LabledImg == i)));
                        if(RegionSize <= MinRegionSize)
                            MinRegionSize = RegionSize;
                            RegionIndex   = i;
                        end

                end
                BinaryImg(find(LabledImg == RegionIndex)) = 0;
 
            end

        end

end

if(Branches > 0)
    warning('The input shape had %d branches. They were removed.',Branches); 
end