function maximize(h)

% MAXIMIZE   maximize a figure window
%    |====================================================================|
%    |                                                                    |
%    |       Berne University of Applied Sciences                         |
%    |                                                                    |
%    |       School of Engineering and Information Technology             |
%    |       Division of Electrical- and Communication Engineering        |
%    |                                                                    |
%    |====================================================================|
%    |                     maximize a figure window                       |
%    |====================================================================|
%    |                                                                    |
%    | Author:    Alain Trostel                                           |
%    | e-mail:    alain.trostel@bfh.ch                                    |
%    | Date:      March 2006                                              |
%    | Version:   3.1                                                     |
%    |                                                                    |
%    |====================================================================|
%    |                                                                    |
%    | function maximize(h)                                               |
%    |                                                                    |
%    | input parameters:                                                  |
%    | -----------------                                                  |
%    | h             handle(s) of the figure window                       |
%    |                                                                    |
%    |                                                                    |
%    | output parameters:                                                 |
%    | ------------------                                                 |
%    | The function has no output parameters.                             |
%    |                                                                    |
%    |                                                                    |
%    | used files:                                                        |
%    | -----------                                                        |
%    | - windowMaximize.dll                                               |
%    |                                                                    |
%    |                                                                    |
%    | examples:                                                          |
%    | ---------                                                          |
%    | % maximize the current figure                                      |
%    | maximize;                                                          |
%    |                                                                    |
%    | % maximize the current figure                                      |
%    | maximize(gcf);                                                     |
%    |                                                                    |
%    | % maximize the specified figure                                    |
%    | h = figure;                                                        |
%    | maximize(h);                                                       |
%    |                                                                    |
%    | % maximize the application window                                  |
%    | maximize(0);                                                       |
%    |                                                                    |
%    | % maximize more than one figure                                    |
%    | h(1) = figure;                                                     |
%    | h(2) = figure;                                                     |
%    | maximize(h);                                                       |
%    |                                                                    |
%    | % maximize all figures                                             |
%    | maximize('all');                                                   |
%    |                                                                    |
%    |====================================================================|

% check if dll-file exists
if ~exist('windowMaximize.dll','file')
    error('windowMaximize.dll not found.');
end

% if no input parameters, get handle of the current figure
if nargin == 0
    h = gcf;
end

% if one input parameter, check the input parameter
if ischar(h)
    % check the string
    if strcmpi(h,'all')
        % get all figure handles
        h = findobj('Type','figure');            
    else
        % incorrect string argument
        error('Argument must be the correct string.');
    end
else
    % check each handle
    for n=1:length(h)
        % it must be a handle and of type 'root' or 'figure'
        if ~ishandle(h(n)) || (~strcmp(get(h(n),'Type'),'root') && ...
                               ~strcmp(get(h(n),'Type'),'figure'))
            % incorrect handle
            error('Argument(s) must be a correct handle(s).');
        end
    end
end

% if handle is not the root
if h ~= 0
    % for each handle
    for n=length(h):-1:1
        % create the temporary window name
        windowname = ['maximize_',num2str(h(n))];

        % save current window name
        numTitle = get(h(n),'NumberTitle');
        figName = get(h(n),'Name');

        % set the temporary window name
        set(h(n),'Name',windowname,'NumberTitle','off');

        % draw figure now
        drawnow;
        % maximize the window with the C function
        windowMaximize(windowname);

        % reset the window name
        set(h(n),'Name',figName,'NumberTitle',numTitle);
    end
else
    % maximize the application window "MATLAB"
    windowMaximize('MATLAB');
end