function [lengthHypo,lengthRoot]=measureHypo1(inputFile,thStruct,markerColor)
% to run: [lengthHypo]=measureHypo4('DSCN4340.tif',2)
% to run: [lengthHypo]=measureHypo3('DSCN4340.tif',2)
% to run: [lengthHypo]=measureHypo3('DSCN4333.tif',2)
% to run: [lengthHypo]=measureHypo3('DSCN4334.tif',1)
% to run: [lengthHypo]=measureHypo1(5,2)
% to run: [lengthHypo]=measureHypo1('DSCN4331.tif')

% in v.11 I transfer the thresholds
% in v.10 I use the calibration file
% in v.8 we use two iterative segmentations for both the hypos and roots

%% general directions

warning off

%% load image

try,
    a=imread(inputFile); % training set, test run (335/7 has 2 color problem, 336 is empty, 338g is very nice)
catch
    figure
    text(0.3,0.5,'no such image', 'FontSize',22)
    grid off
    beep
    return
end

% g: 330, 331, 338, 339, 340, 342,
% r: 329, 341, 334
% both: 335, 337
% empty: 336, 343

%% parameters: thresholds, colors etc.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% extract the thresholds from the struct
BWTh=thStruct.hyposBW1;
BWSecondThHypos=thStruct.hyposBW2;
BWThRoots=thStruct.rootsBW1;
BWSecondThRoots=thStruct.rootsBW2;
showValues=thStruct.showValues;


% old "effective" area conversion
% conversionRatio=111/905*(1/0.7);

% manual calibration shows that 243-152=91 pixels is the grid - 13mm
% conversionRatio=13/89;

try
    load('calibrationRatioFile.mat')
    conversionRatio=calibrationRatio;
catch
    conversionRatio=13/95; % full scale exp
    disp('Warning: there is no calibration file in the current directory, we proceed with a default setting')
end


% choose color of markings

% markingsColor=1; % red
% markingsColor=2; % green
% markingsColor=3; % blue
% markingsColor=4; % both red and green

% this governs whether to show the original and final image
showProcessed=0;

% the default markings color is green
markingsColor=2;

% check is
if nargin>1,
    markingsColor=markerColor;
end

% markingsThRed=70; % original red th
% markingsThRed=65; % original red th - was optimal for first two plates
markingsThRed=50; % original red th

% markingsThGreen=30; % original green th - was optimal for first two plates
markingsThGreen=20; % original green th

markingsThBlue=10; % original blue th

showIntermediate=0;
% showIntermediate=1;

% this is the threshold that dictates the ratio of x to y distance the is considered one on top of the other

% ratioTh=0.3; % original value used
% ratioTh=0.2; % learning/testing a more robust value for not connecting adjacent seedlings
% ratioTh=0.3; % tried for full test set
ratioTh=0.5; % tried for full test set

%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% crop image

% cropSizeXl=130; % optimized value for test run
cropSizeXl=70; % full scale exp
cropSizeXr=70; % optimized value for test run

cropSizeYl=50; % optimized value for test run
cropSizeYr=50; % optimized value for test run

if showProcessed,
    figure, imshow(a)
    figure, imshow(a)
end;

IFull=rgb2gray(a);
longAxis=size(IFull,2);
shortAxis=size(IFull,1);
ICropped=imcrop(IFull,[cropSizeXl-1 cropSizeYl-1 longAxis-cropSizeXl-cropSizeXr-1 shortAxis-2*cropSizeYl-cropSizeYr-1]);

if showIntermediate,
    figure, imshow(ICropped)
end

I=ICropped;

%% first thresholding

BW=(I>BWTh);
if showIntermediate,
    figure, imshow(BW)
end

%% the main module for measuring hypocotyls, first run

measureHyposModule2;

%% define an area for extra analysis around the existing roots

se = strel('rectangle',[40 20]); % let this be the area around the existing roots

areaAroundHypos = imdilate(morph12,se);

se = strel('line',20,0);

areaFlankingHypos = imdilate(morph12,se);
areaFlankingHypos2=imsubtract(areaFlankingHypos,morph12); % take off the roots themselves

aroundHyposImage=immultiply(I,uint8(areaAroundHypos));
upDownHyposImage=immultiply(~areaFlankingHypos2,aroundHyposImage);

if showIntermediate,
    figure, imshow(areaAroundHypos); maximize
    figure, imshow(aroundHyposImage); maximize
    figure, imshow(upDownHyposImage); maximize
end;

%% start measuring the roots, first thresholding

BW=(upDownHyposImage>BWSecondThHypos);
if showIntermediate,
    figure, imshow(BW)
end

%% the main module for measuring hypocotyls, second run

measureHyposModule2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% start measuring the roots, first thresholding

BWRoots=(I>BWThRoots);
if showIntermediate,
    figure, imshow(BWRoots)
end

%% the main module for measuring roots, first run, use this as the input
%% for the second run

measureRootsModule2;

%% define an area for extra analysis around the existing roots

se = strel('rectangle',[40 20]); % let this be the area around the existing roots

areaAroundRoots = imdilate(morph12Roots,se);

se = strel('line',20,0);

areaFlankingRoots = imdilate(morph12Roots,se);
areaFlankingRoots2=imsubtract(areaFlankingRoots,morph12Roots); % take off the roots themselves

aroundRootsImage=immultiply(I,uint8(areaAroundRoots));
upDownRootsImage=immultiply(~areaFlankingRoots2,aroundRootsImage);

if showIntermediate,
    figure, imshow(areaAroundRoots); maximize
    figure, imshow(aroundRootsImage); maximize
    figure, imshow(upDownRootsImage); maximize
end;

%% start measuring the roots, first thresholding

BWRoots=(upDownRootsImage>BWSecondThRoots);
if showIntermediate,
    figure, imshow(BWRoots)
end

%% the main module for measuring roots, second run

measureRootsModule2;

%% calculate the perimeters

perimeterVectRoots=PerimeterPixelsRoots*conversionRatio;

% these are values up to one digit after decimal point
lengthRootVect=round(10*perimeterVectRoots)/10;

if showValues, % after the calibration you should create a figure here
figure,
end
imshow(uint8(IsumRoots),[0 255]);

% show roots lengths
for loop=1:length(STATSRoots),
    [j,i]=find(L7Roots==loop);
    lengthRoot=lengthRootVect(loop);
    %     text(i(round(length(i)/2))+6,j(round(length(i)/2)),num2str(lengthRoot),'Color','m','FontSize',14)
    if showValues,
        text(i(round(length(i)/2))+6,j(round(length(i)/2)),num2str(lengthRoot),'Color',[1 0.8 0.5],'FontSize',14)
    end
end

% show hypo lengths
for loop=1:length(STATS),
    [j,i]=find(L5==loop);
    % show stem length instead of area, using a simlpe calibration
    lengthStem=lengthStemVect(loop);
    if showValues,
        text(i(round(length(i)/2))+6,j(round(length(i)/2)),num2str(lengthStem),'Color','y','FontSize',14)
    end
end

title('absolute calibration');

% disp(lengthRootVect')

lengthRoot=lengthRootVect';

% save figure
saveas(gcf,[inputFile,'.bmp'],'bmp')

% save database of analysis for possible debuging later
save debugPoint


