%% first skeletonization
BW3Roots = bwmorph(BWRoots,'skel',Inf);
if showIntermediate,
    figure, imshow(BW3Roots)
end

% go around the closing and opening operation
% omit the hypocotyls already detected
morph2Roots=BW3Roots.*logical(~L6);
if showIntermediate,
    figure, imshow(morph2Roots);
end

%% find objects, filter out minimal box size and minimal area
morph6Roots=morph2Roots; % go around the opening, closing and skeleton

LRoots = bwlabel(morph6Roots,8);
STATSRoots = regionprops(LRoots,'Area','Centroid','BoundingBox');
Centroid=cell2mat({STATSRoots.Centroid});
area=cell2mat({STATSRoots.Area});

% this enables us to find objects that are straight lines and eliminte them
% they are recognized by being 1 pixels width in some dimension throughout
boundingBox=cell2mat({STATSRoots.BoundingBox}');
if ~isempty(boundingBox),
    boxXWidth=boundingBox(:,3);
    boxYWidth=boundingBox(:,4);
else
    boxXWidth=[];
    boxYWidth=[];
end


minAreaTh=20; % important th - original value
% minAreaTh=10; % important th - new value

minBoxYWidthTh=2;
minBoxYWidthTh=10;

% don't take things with location above the hypos
minYLoc=mean2(yMat1);
maxYLoc=size(LRoots,1)-50;

%% find centroid locations, define and filter "bad" objects

if ~isempty(boundingBox),
    yLoc=Centroid(1,2:2:end);
else
    yLoc=[];
end

% badObjects=find(area'<minAreaTh | boxXWidth<2 | boxYWidth<2);
badObjects=find(area'<minAreaTh | boxXWidth<2 | boxYWidth<minBoxYWidthTh...
    | yLoc'<minYLoc+20 | yLoc'>maxYLoc);


morph7Roots=morph6Roots;
for loop=1:length(badObjects),
    morph7Roots(find(LRoots==badObjects(loop)))=0;
end

if showIntermediate,
    figure, imshow(morph7Roots)
end

morph8Roots = bwmorph(morph7Roots,'skel',Inf);
LRoots = bwlabel(morph8Roots,8);
STATSRoots = regionprops(LRoots,'Area','Centroid');
Centroid=cell2mat({STATSRoots.Centroid});

% showIntermediate=1;
if showIntermediate,
    figure, imagesc(LRoots), maximize, title('objects before intersection')
end

%% subtract segmented from markers, subtract marks from image, find objects
oldMark=BW3;

morph10Roots=morph8Roots-immultiply(morph8Roots,oldMark);

% morph10=morph6-immultiply(morph6,intersectI3);

if showIntermediate,
    %     figure, imshow(morph8Roots)
    figure, imshow(morph10Roots)
end

L4Roots = bwlabel(morph10Roots,8);
if showIntermediate,
    figure, imagesc(L4Roots)
end


%% collect data, filter by minimal area
STATSRoots = regionprops(L4Roots,'Area','Centroid');
area=cell2mat({STATSRoots.Area});

% minAreaTh=50; % original value
% minAreaTh=10;
minAreaTh=15; % that will be equivalent to a length of about 2.2 mm
minAreaTh=30; % I try a higher value so as not to have very short roots

maxAreaTh=300; % A suggestion for an upper value
% maxAreaTh=2000; % A suggestion for an upper value

badObjects=find(area'<minAreaTh | area'>maxAreaTh);
% badObjects=find(area'<minAreaTh);
morph11Roots=morph10Roots;
for loop=1:length(badObjects),
    morph11Roots(find(L4Roots==badObjects(loop)))=0;
end

if showIntermediate,
    %     figure, imshow(morph8Roots)
    figure, imshow(morph11Roots)
end

L5Roots = bwlabel(morph11Roots,8);
STATSRoots = regionprops(L5Roots,'Area','Centroid');
area=cell2mat({STATSRoots.Area});
if showIntermediate,
    figure, imagesc(L5Roots)
    figure, imshow(logical(L6)+logical(L5Roots))
end




%% keep only objects that touch hypocotyls detected previously

mergedImageRoots=zeros(size(L5Roots)); % this will be only the things that touch the hypo
for loop=1:length(STATSRoots),
    subLRoots=zeros(size(L5Roots));
    subLRoots(L5Roots==loop)=1;
    se = strel('diamond',20);
    %     se = strel('rectangle',[40 20])
    subL2Roots = imdilate(subLRoots,se);
    intersectImageRoots=logical(L6).*subL2Roots;
    numberPixelsIntersectingRoots=length(find(intersectImageRoots));
    if numberPixelsIntersectingRoots>0,
        mergedImageRoots=mergedImageRoots+subLRoots;
    end
end;

if showProcessed,
    figure, imshow(mergedImageRoots)
    figure, imshow(subL2Roots)
end

if showIntermediate,
    figure, imshow(intersectImageRoots), maximize, title('objects after intersection')
end
morph11Roots=mergedImageRoots;
%%

morph12Roots = imfill(morph11Roots,'holes');

% L7Roots = bwlabel(morph11Roots,8);
L7Roots = bwlabel(morph12Roots,8);
if showIntermediate,
    figure, imshow(morph11Roots)
    figure, imagesc(L7Roots)
end
STATSRoots = regionprops(L7Roots,'Area','Centroid');

%% dilate shoots
mask2Roots=ICropped;
se = strel('line',5,0);
L6Roots = imdilate(L7Roots,se);
mask2Roots(L6Roots>0)=0;

if showIntermediate,
    figure, imshow(logical(L6Roots));
end

if showIntermediate,

    figure, imshow(morph12Roots);
    figure, imshow(L6Roots);
    figure, imshow(BW3);
    figure, imshow(mask2);
    figure, imshow(mask2Roots);
    figure, imshow(ICropped);

end
%% create an overlay image, add sizes

% new
% hypoImage=double(uint8(128*double(LMeasuredHypoDilated)));
% rootsImage=double(uint8(128*double(L6Roots)));

backgroundImage=double(uint8((ICropped)));

if showIntermediate,
    figure, imshow(backgroundImage); colorbar, maximize
    %     figure, imshow(hypoImage); colorbar; maximize
    %     figure, imshow(IsumRoots(:,:,1),[0 255]); maximize
    %     figure, imshow(uint8(IsumRoots),[0 255]); maximize
    % %     figure, imshow(rootsImage); maximize
    %     figure, imshow(uint8((ICropped))); maximize
end

% clear old data just in case
clear PerimeterPixels
PerimeterPixelsRoots=[];

measuredRootsMask=zeros(size(L7Roots));
for loop=1:length(STATSRoots),
    [j,i]=find(L7Roots==loop);
    oneRootImage=zeros(size(L7Roots));
    oneRootImage(find(L7Roots==loop))=1;
    try,
        [PerimeterPixelsRoots(loop),measuredRoot] = EstimatePerimeterRon(oneRootImage);
        measuredRootsMask=measuredRootsMask+measuredRoot(:,:,1);
    catch,
        PerimeterPixelsRoots(loop) = 0; % if there is an error in measuring the length - quite rare
    end;
end

%%

measuredRootsMask=uint8(measuredRootsMask);
LMeasuredRoots = bwlabel(measuredRootsMask,8);

se = strel('line',5,0);
LMeasuredRootsDilated = imdilate(LMeasuredRoots,se);

rootsImage=double(uint8(128*double(LMeasuredRootsDilated)));
hypoImage=double(uint8(128*double(LMeasuredHypoDilated)));

% IsumRoots(:,:,1)=double(max(0,backgroundImage+uint8(128*double(oldMark))-rootsImage-hypoImage));
IsumRoots(:,:,1)=double(max(0,backgroundImage+(128*double(oldMark))-rootsImage-hypoImage));
IsumRoots(:,:,2)=double(min(backgroundImage+hypoImage,255));
IsumRoots(:,:,3)=double(min(backgroundImage+rootsImage,255));

